<?php

namespace App\Core;

/**
 * A bit of a janky way to display a custom page on error or exception.
 * But looks pretty cool and professional!
 */
class ErrorHandler 
{
    public array $error_messages = [];

    public function __construct()
    {
        // if an error were to occur, we clear the outputted data so we only show the error page instead of both
        ob_Start();

        set_error_handler([$this, 'error']);
        set_exception_handler([$this, 'exception']);
        register_shutdown_function([$this, 'shutdown']);
    }

    public function error($errno, $errstr, $errfile, $errline): void
    {
        $errstr = htmlspecialchars($errstr);
        $this->error_messages[] = "<b>Error[$errno]:</b> $errstr in <b>$errfile</b> at line <b>$errline</b>";
    }

    public function exception($exception): void
    {
        $this->error_messages[] = "<b>Uncaught Exception:</b> " . $exception;
    }

    public function shutdown(): void
    {
        if (!$this->error_messages) {
           return; 
        }

        // remove current output to be replaced by error page
        ob_end_clean();

        http_response_code(500);
        header_remove();

        echo '<style>';
        echo '
        body {
            margin: 1rem;
            background: tomato;
            font-family: sans-serif;
        }
        #error {
            padding: 1rem;
            background: white;
            margin-bottom: 1rem;
            border: .1rem solid black;
        }
        ';
        echo '</style>';
        echo '<h1>Error!!1 (✖﹏✖)</h1>';
        echo '<p>Oisann! Dette var ikke ment å skje. Dersom det vedvarer, vennligst kontakt nettadministratoren.</p>';

        foreach ($this->error_messages as $error_message) {
           echo "<div id=\"error\">$error_message</div>";
        }
    }
}